/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s): 
 *   Daniel Wippermann <daniel@wippermann.net>
 */

#include "nsSocketListenerProxy.h"
#include "nsISocketTransport.h"
#include "nsIEventQueueService.h"
#include "nsIServiceManager.h"

static NS_DEFINE_CID(kEventQueueServiceCID, NS_EVENTQUEUESERVICE_CID);

NS_IMPL_ISUPPORTS2 (nsSocketListenerProxy,
                    nsISocketListener,
                    nsISocketListenerProxy);

class nsOnSocketAcceptedEvent
{
public:
    nsOnSocketAcceptedEvent ()
    {
        MOZ_COUNT_CTOR (nsOnSocketAcceptedEvent);
    }

    ~nsOnSocketAcceptedEvent ()
    {
        MOZ_COUNT_DTOR (nsOnSocketAcceptedEvent);
    }

    nsresult FireEvent (nsISocketListener* aListener,
                        nsIEventQueue* aEventQ,
                        nsISupports* aContext,
                        nsISocketTransport* aSocket);

    void HandleEvent ();

    static nsOnSocketAcceptedEvent* FromEvent (PLEvent* aEvent)
    {
        return (nsOnSocketAcceptedEvent*) ((char*) aEvent -
            offsetof (nsOnSocketAcceptedEvent, mEvent));
    };

    static void* PR_CALLBACK StaticHandleEvent (PLEvent* aEvent)
    {
        FromEvent (aEvent)->HandleEvent ();
        return nsnull;
    }
    
    static void PR_CALLBACK StaticDestroyEvent (PLEvent* aEvent)
    {
        delete FromEvent (aEvent);
    }

protected:
    PLEvent mEvent;
    nsCOMPtr <nsISocketListener> mListener;
    nsCOMPtr <nsISupports> mContext;
    nsCOMPtr <nsISocketTransport> mSocket;
};


nsresult
nsOnSocketAcceptedEvent::FireEvent (nsISocketListener* aListener,
                                    nsIEventQueue* aEventQ,
                                    nsISupports* aContext,
                                    nsISocketTransport* aSocket)
{
    mListener = aListener;
    if (!mListener) return NS_ERROR_INVALID_ARG;
    mSocket = aSocket;
    if (!mSocket) return NS_ERROR_INVALID_ARG;
    mContext = aContext;

    PL_InitEvent (&mEvent, nsnull, StaticHandleEvent, StaticDestroyEvent);

    return aEventQ->PostEvent (&mEvent);
}


void
nsOnSocketAcceptedEvent::HandleEvent ()
{
    mListener->OnSocketAccepted (mContext, mSocket);
}


NS_IMETHODIMP
nsSocketListenerProxy::OnSocketAccepted (nsISupports* aContext,
                                         nsISocketTransport* aSocket)
{
    nsOnSocketAcceptedEvent* Event = new nsOnSocketAcceptedEvent ();
    if (!Event) return NS_ERROR_OUT_OF_MEMORY;
    return Event->FireEvent (mListener, mEventQ, aContext, aSocket);
}


NS_IMETHODIMP
nsSocketListenerProxy::Init (nsISocketListener* aListener,
                             nsIEventQueue* aEventQ)
{
    nsresult rv = NS_OK;
    
    mListener = aListener;

    if (aEventQ == NS_CURRENT_EVENTQ) {
        nsCOMPtr <nsIEventQueueService> eqs = do_GetService (kEventQueueServiceCID, &rv);
        if (NS_FAILED (rv)) return rv;
        rv = eqs->GetSpecialEventQueue ((PRInt32) aEventQ, getter_AddRefs (mEventQ));
    } else
        mEventQ = aEventQ;

    return rv;
}
